import sys, os
import xbmc, xbmcgui, xbmcplugin
import urllib

LOG_LEVEL = 2

# plugin modes
MODE_SCAN_SOURCE = 10
MODE_SHOW_SOURCES = 20
MODE_HELP = 30

# parameter keys
PARAMETER_KEY_MODE = "mode"
PARAMETER_KEY_SOURCE = "source"

# menu item names
MENU_SHOW_SOURCE = "Show Source Paths"
HELP_SUBMENU = "About"

def log(line, logLevel):
    if logLevel <= LOG_LEVEL:
        print "Missing Movie Scanner: " + line

def remove_duplicates(files):
    # converting it to a set and back drops all duplicates
    return list(set(files))

def clean_name(text):
    text = text.replace('%21', '!')
    text = text.replace('%3a', ':')
    text = text.replace('%5c', '\\')
    text = text.replace('%2f', '/')
    text = text.replace('%2c', ',')

    return text

def get_movie_sources():    
    log("get_movie_sources() called", 3)
    
    jsonResult = xbmc.executeJSONRPC('{"jsonrpc": "2.0", "method": "Files.GetSources", "params": {"media": "video"}, "id": 1}')
    log("VideoLibrary.GetSources results:\n" + jsonResult, 4)
    shares = eval(jsonResult)['result']['shares']
    shares = [ s['file'] for s in shares ]

    results = []
    for s in shares:
        log("FOUND SHARE: " + s, 1)
        if s.startswith('addons://'):
            shares.remove(s)
        elif s.startswith('multipath://'):
            s = s.replace('multipath://', '')
            parts = s.split('/')
            parts = [ clean_name(f) for f in parts ]

            for b in parts:
                if b:
                    results.append(b)
        else:
            results.append(s)

    return results

    
# plugin handle
handle = int(sys.argv[1])

FILE_EXTENSIONS = ['mpg', 'mpeg', 'avi', 'flv', 'wmv', 'mkv', '264', '3g2', '3gp', 'ifo', 'mp4', 'mov', 'iso', 'ogm']
FILE_EXTENSIONS.extend(xbmcplugin.getSetting(handle, "custom_file_extensions").split(";"))

def file_has_extensions(filename, extensions):
    # get the file extension, without a leading colon.
    name, extension = os.path.splitext(os.path.basename(filename))
    name = name.lower()
    extension = extension[1:].lower()
    extensions = [ f.lower() for f in extensions ]

    if extension == 'ifo' and name != 'video_ts':
        return False

    return extension in extensions

def get_files(path):
    log("get_files(path) called", 3)
    os_name = os.name
    log(os_name, 1)
    smbProt = False
    
	# and os_name.lower() == "nt":
    if path.startswith("smb:"):
	    log("Is Windows and has SMB in path", 1)
	    smbProt = True
	    path = path.replace('smb:', '')

    log("WALKING PATH : " + path, 2)
    results = []
    
    for root, sub_folders, files in os.walk(path):
        log("PROCESSING PATH : " + root, 3)
        for f in files:
            log("PROCESSING FILE : " + f, 3)
            if file_has_extensions(f, FILE_EXTENSIONS):
                if smbProt:
                    f = "smb:" + root + "/" + f
                else:
                    f = os.path.join(root, f)
                log("ADDING FILE : " + f, 3)
                results.append(f)

    return results

#############################################################################################
# utility functions
#############################################################################################
def parameters_string_to_dict(parameters):
    ''' Convert parameters encoded in a URL to a dict. '''
    paramDict = {}
    if parameters:
        paramPairs = parameters[1:].split("&")
        for paramsPair in paramPairs:
            paramSplits = paramsPair.split('=')
            if (len(paramSplits)) == 2:
                log("Param " + paramSplits[0] + "=" + paramSplits[1], 3)
                paramDict[paramSplits[0]] = paramSplits[1]
    return paramDict

def addDirectoryItem(name, isFolder=True, parameters={}, totalItems=1):
    ''' Add a list item to the XBMC UI.'''
    li = xbmcgui.ListItem(name)

    url = sys.argv[0] + '?' + urllib.urlencode(parameters)

    if not isFolder:
        url = name
        
    log("Adding Directory Item: " + name + " totalItems:" + str(totalItems), 2)
    
    return xbmcplugin.addDirectoryItem(handle=handle, url=url, listitem=li, isFolder=isFolder, totalItems=totalItems)

#############################################################################################
# UI builder functions
#############################################################################################
def show_root_menu():
    ''' Show the plugin root menu. '''
    log("LOG LEVEL : " + str(LOG_LEVEL), 0)
    addDirectoryItem(name=MENU_SHOW_SOURCE, parameters={ PARAMETER_KEY_MODE: MODE_SHOW_SOURCES }, isFolder=True)
    addDirectoryItem(name=HELP_SUBMENU, parameters={ PARAMETER_KEY_MODE: MODE_HELP }, isFolder=True)
    xbmcplugin.endOfDirectory(handle=handle, succeeded=True)
    
def scan_movie_source(source_path):
    ''' Show movies missing from the library. '''
    log("scan_movie_source(source_path) : " + source_path, 2);
        
    jsonResult = xbmc.executeJSONRPC('{"jsonrpc": "2.0", "method": "VideoLibrary.GetMovies", "params": {"fields": ["file", "trailer"]}, "id": 1}')
    log("VideoLibrary.GetMovies results: " + jsonResult, 4)
    result = eval(jsonResult)
    movies = result['result']['movies']

    library_files = []
    missing = []

    # add files from trailers and sets
    for m in movies:
        f = m['file']

        if f.startswith("videodb://"):
            set_files = eval(xbmc.executeJSONRPC('{"jsonrpc": "2.0", "method": "Files.GetDirectory", "params": {"directory": "%s"}, "id": 1}' % f))

            sub_files = []
            sub_trailers =  []

            for item in set_files['result']['files']:
                sub_files.append(clean_name(item['file']))
                try:
                    trailer = item['trailer']
                    if not trailer.startswith('http://'):
                        library_files.append(clean_name(trailer))
                except KeyError:
                    pass

            library_files.extend(sub_files)
            library_files.extend(sub_trailers)
        elif f.startswith('stack://'):
            f = f.replace('stack://', '')
            parts = f.split(' , ')

            parts = [ clean_name(f) for f in parts ]

            for b in parts:
                library_files.append(b)
        else:
            library_files.append(clean_name(f))
            try:
                trailer = m['trailer']
                if not trailer.startswith('http://'):
                    library_files.append(clean_name(trailer))
            except KeyError:
                pass

    library_files = set(library_files)

    movie_files = set(get_files(source_path))
    
    if not library_files.issuperset(movie_files):
        log("Adding missing library items to list for souce path: " + source_path, 1)
        missing.extend(list(movie_files.difference(library_files)))

        for movie_file in missing:
            # get the end of the filename without the extension
            if os.path.splitext(movie_file.lower())[0].endswith("trailer"):
                log(movie_file + " is a trailer and will be ignored!", 1) 
            else:
                log("Adding missing item: " + movie_file, 1)
                addDirectoryItem(movie_file, isFolder=False, totalItems=len(missing))

    else:
        addDirectoryItem("No Missing Items", isFolder=False, totalItems=1)
                
    xbmcplugin.endOfDirectory(handle=handle, succeeded=True)
    
def show_source_list():
    ''' Show movies missing from the library. '''
    MOVIE_PATHS = remove_duplicates(get_movie_sources())
    
    for movie_path in MOVIE_PATHS:
        addDirectoryItem(movie_path, parameters={ PARAMETER_KEY_MODE: MODE_SCAN_SOURCE, PARAMETER_KEY_SOURCE: movie_path }, isFolder=True, totalItems=len(MOVIE_PATHS))
        
    xbmcplugin.endOfDirectory(handle=handle, succeeded=True)

def show_about():
    xbmcgui.Dialog().ok("About", "Compares your source paths to your library.", 
        "Lists missing movies from the library.", 
        "You can add custom file extensions in the settings.")

# parameter values
params = parameters_string_to_dict(sys.argv[2])
mode = int(urllib.unquote_plus(params.get(PARAMETER_KEY_MODE, "0")))
source = urllib.unquote_plus(params.get(PARAMETER_KEY_SOURCE, ""))
LOG_LEVEL = int(xbmcplugin.getSetting(handle, "custom_log_level"))

# Depending on the mode, call the appropriate function to build the UI.
if not sys.argv[2]:
    # new start
    ok = show_root_menu()
elif mode == MODE_SCAN_SOURCE:
    ok = scan_movie_source(source)
elif mode == MODE_SHOW_SOURCES:
    ok = show_source_list()
elif mode == MODE_HELP:
    ok = show_about()
